#!/usr/bin/env python
# GNU GPL

import string
import GameList
import Config
import Games
import Selects
import Show
import Filter
import Preferences
import View
import gettext
import pexpect

import pygtk
pygtk.require('2.0')
import gtk
import gobject

class Main(gobject.GObject):
    def __init__(self):
    
        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        window.set_default_size(800,700)
        window.connect("delete_event", self.delete_event)
        window.set_border_width(10)
        window.set_title("PyXmame")
        
        gettext.install('pyxmame','../locale',codeset='ISO-8859-15')

        self.config=Config.Config()
        self.config.read()
        
        self.gamelist=GameList.GameList(self.config)
        
        self.gamelist.load_list()
        
        ### selects
        self.selects_box = Selects.Selects(
                                    self.config,
                                    self.gamelist.get_lists())

        ### games list
        self.list_box = Games.Games(  
                                    self.config,
                                    self.start_xmame,
                                    self.show_game)

        ### show games
        self.game_box=Show.Show(self.config)
        
        ### log game execution
        self.view_box=View.View()

        b1 = gtk.Button(stock=gtk.STOCK_GO_FORWARD)
        b1.connect("clicked", self.select_button, None)
        alignment = b1.get_children()[0]
        hbox = alignment.get_children()[0]
        image, label = hbox.get_children()
        label.set_text(_('Filter'))
        
        b2 = gtk.Button(stock=gtk.STOCK_FIND)
        b2.connect("clicked", self.filter_dialog, None)

        b3 = gtk.Button(stock=gtk.STOCK_PREFERENCES)
        b3.connect("clicked", self.preferences_dialog, None)

        left_box = gtk.VBox(False, 10)
        left_box.pack_start(self.selects_box, True, True, 0)
        left_box.pack_start(b1, False, True, 0)
        left_box.pack_start(b2, False, True, 0)
        left_box.pack_start(b3, False, True, 0)

        up_box = gtk.HBox(False, 10)
        up_box.pack_start(left_box, False, True, 0)
        up_box.pack_start(self.list_box, True, True, 0)
        up_box.pack_start(self.view_box, True, True, 0)
        
        vbox = gtk.VBox(False, 10)
        vbox.pack_start(up_box, True, True, 0)
        vbox.pack_start(self.game_box, False, True, 0)
        window.add(vbox)

        window.connect("key-press-event",self.key_press_event)
        
        window.show_all()
        self.view_box.hide_all()
        self.selects_box.change_select(None)
        
        #window.maximize()
        
    def start_xmame(self,treeview,path,view_column):
        self.list_box.hide()
        self.view_box.show_all()
        
        argv=list()
        argv.append(self.config["binary"])
        for extra in self.config["extra_param"].split(" "):
            argv.append(extra)
        argv.append(self.config["rompath_param"])
        argv.append(self.config["rompath"])
        argv.append(self.list_box.get_selection())
        
        self.view_box.insert(string.join(argv," ")+"\n")
        child = pexpect.spawn (string.join(argv," "))
        
        pattern = "([^\n]+)\n"
        i = child.expect ([pattern, pexpect.EOF])
        while i == 0:
            line=child.match.groups()[0]
            self.view_box.insert(line)
            i = child.expect ([pattern, pexpect.EOF],timeout=None)

        self.view_box.hide()
        self.list_box.show_all()
        self.view_box.initialize()
        self.list_box.set_focus()
        return

    def show_game(self,treeview):
        game=self.list_box.get_selection()
        if game == None:
            return
        self.game_box.set_game(self.gamelist.list[game])
        return

    def delete_event(self, widget, event, data=None):
        dlg = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION)
        dlg.set_markup(_("Exit now ?"))
        dlg.add_button(gtk.STOCK_QUIT,gtk.RESPONSE_OK)
        dlg.add_button(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL)
        response = dlg.run()
        dlg.destroy()
        if response != gtk.RESPONSE_OK:
            return True
        gtk.main_quit()
        return True
       
    def filter_dialog(self,widget=None,event=None):
        dlg = Filter.Filter(self.config)
        if dlg.run() != gtk.RESPONSE_ACCEPT:
            dlg.destroy()
            return
        patterns=dlg.get_values()
        dlg.destroy()
        match=self.gamelist.get_list(patterns)
        self.list_box.set_list(match)
        self.selects_box.initialize()
        return

    def preferences_dialog(self,widget=None,event=None):
        dlg = Preferences.Preferences()
        dlg.run()
        self.config.read()
        if dlg.gamelist_builded:
            self.gamelist.load_list()
        dlg.destroy()
        return

    def select_button(self,widget=None,event=None):
        pattern=self.selects_box.get_selection()
        if pattern == None:
            self.list_box.set_focus()
            return
        match=self.gamelist.get_list(pattern)
        self.list_box.set_list(match)
        return

    def key_press_event(self,event,key):
        key=gtk.gdk.keyval_name(key.keyval)
        if key == "Escape":
            self.delete_event(self, None, None)
            return True
        elif key == "f":
            self.filter_dialog()
            return True
        elif key == "Right":
            self.select_button()
            return True
        elif key == "Left":
            self.selects_box.set_focus()
            return True
        return
        
    def main(self):
        gtk.main()
        return

if __name__ == "__main__":
    peu = Main()
    peu.main()
