#!/usr/bin/env python
# GNU GPL

import string
import GameList
import Config
import gettext
import sys,os

import pygtk
pygtk.require('2.0')
import gtk
import gobject


class Fields(gtk.VBox):

    def __init__(self):
        gtk.VBox.__init__(self,False,20)
        self.values= dict()
        self.config=Config.Config()
        self.config.read()
        for entry in self.list:
            textview=gtk.Label(_(self.query_text_fields[entry]))
            textview.set_alignment(0,1)
        
            self.values[entry]=gtk.Entry()
            self.values[entry].set_max_length(200)
            #self.filter[entry].connect("activate", filter_games)
            self.values[entry].select_region(0, len(self.values[entry].get_text()))
            self.values[entry].connect("insert-text",self.set_text,entry)
            self.values[entry].set_text(self.config[entry])

            button=gtk.Button(stock=gtk.STOCK_OPEN)
            button.connect("clicked",self.file_selection,entry)

            values_box=gtk.HBox(False,10)
            values_box.pack_start(self.values[entry])
            values_box.pack_start(button,False)

            entry_box=gtk.VBox(False,0)
            entry_box.pack_start(textview)
            entry_box.pack_start(values_box)
            self.pack_start(entry_box)
        return

    def set_text(self,widget,text,text_len,pos,entry):
        #~ if text_len == 1:
            #~ self.labels[entry].set_text(self.values[entry].get_text())
        #~ else:
            #~ self.labels[entry].set_text(text)
        return
        
    def get_text(self,key):
        return self.values[key].get_text()

    def file_selection(self,widget,entry):
        selection=gtk.FileSelection(self.file_selec_titles[entry])
        selection.hide_fileop_buttons()
        selection.show_all()
        response = selection.run()
        if response != gtk.RESPONSE_OK:
            selection.destroy()
            return
        self.values[entry].set_text(selection.get_selections()[0])
        selection.destroy()
        return
        
    def get_label(self):
        return self.label

class DirectoriesFields(Fields):
    query_text_fields={ "snaps":    "Choose directory or zip file where the snapshots are located",
                        "flyers":   "Choose directory or zip file where the flyers are located",
                        }
    file_selec_titles={ "snaps":    "Select snapshots",
                        "flyers":   "Select flyers",
                        }
    list=["snaps","flyers"]
    def __init__(self):
        Fields.__init__(self)
        self.label=_("Directories")
        return

class GameListFields(Fields):
    query_text_fields={ "rompath":  "Choose directory where the roms are located",
                        "catver":   "Choose catver.ini file"
         }
    file_selec_titles={ "rompath":  "Select rompath",
                        "catver":   "Select catver.ini file"
         }
    list=["rompath","catver"]
    def __init__(self):
        Fields.__init__(self)
        self.label=_("GameList")
        return


class Page(gtk.VBox):
    def __init__(self,fields,buttons):
        gtk.VBox.__init__(self)
        self.set_border_width(10)
        self.pack_start(fields,False)
        self.pack_start(gtk.HBox(),True)
        self.pack_start(buttons,False)
        self.fields=fields
        return
        
    def get_label(self):
        return self.fields.get_label()
        
    def get_text(self,key):
        return self.fields.get_text(key)
        
    def get_values(self):
        values=list()
        for key in self.fields.list:
            values.append((key,self.fields.get_text(key)))
        return values

class Preferences(gtk.Dialog):

    def __init__(self):
        gettext.install('pyxmame','../locale',codeset='ISO-8859-15')

        gtk.Dialog.__init__(self,title=_("Preferences"), parent=None, flags=gtk.DIALOG_MODAL)

        self.connect("delete_event", self.delete_event)
        self.connect("key-press-event",self.key_press_event)
        self.set_border_width(10)
        self.set_resizable(False)
        self.set_size_request(600,400)

        self.config=Config.Config()
        self.config.read()
        self.gamelist_builded=False

        self.page=list()
        self.page.append(Page(DirectoriesFields(),self.directory_buttons()))
        self.page.append(Page(GameListFields(),self.gamelist_buttons()))

        self.notebook=gtk.Notebook()
        self.notebook.set_tab_pos(gtk.POS_TOP)
        #self.notebook.set_border_width(10)
        self.notebook.append_page(self.page[0],gtk.Label(self.page[0].get_label()))
        self.notebook.append_page(self.page[1],gtk.Label(self.page[1].get_label()))
        #self.notebook.connect("switch-page",self.switch_page)

        self.vbox.add(self.notebook)
        self.show_all()

    def write_config(self,widget=None,event=None):
        if not self.valid():
            return
        for key,value in self.page[0].get_values():
            if value.strip() != "":
                self.config[key]=value
        self.config.write()
        self.config.read()
        return

    def write_gamelist(self,widget=None,event=None):
        if not self.valid():
            return
        self.config.read()
        for key,value in self.page[1].get_values():
            if value.strip() != "":
                self.config[key]=value
        self.config.write()
        self.gamelist=GameList.GameList(self.config)
        self.gamelist.build_list()
        self.gamelist_builded=True
        self.config.read()
        return


    def directory_buttons(self):
        b1 = gtk.Button(stock=gtk.STOCK_CANCEL)
        b1.connect("clicked", self.delete_event, None)

        b2 = gtk.Button(stock=gtk.STOCK_APPLY)
        b2.connect("clicked", self.write_config, None)
        alignment = b2.get_children()[0]
        hbox = alignment.get_children()[0]
        image, label = hbox.get_children()
        label.set_text(_('Save'))


        button_box=gtk.HBox(False, 10)
        button_box.pack_start(b1,False)
        button_box.pack_start(gtk.HBox(),True)
        button_box.pack_start(b2,False)
        return button_box

    def gamelist_buttons(self):
        b1 = gtk.Button(stock=gtk.STOCK_CANCEL)
        b1.connect("clicked", self.delete_event, None)
        b2 = gtk.Button(stock=gtk.STOCK_APPLY)
        b2.connect("clicked", self.write_gamelist, None)
        alignment = b2.get_children()[0]
        hbox = alignment.get_children()[0]
        image, label = hbox.get_children()
        label.set_text(_('Build Gamelist'))


        button_box=gtk.HBox(False, 10)
        button_box.pack_start(b1,False)
        button_box.pack_start(gtk.HBox(),True)
        button_box.pack_start(b2,False)
        return button_box

    def valid(self):
        return True

    def delete_event(self, widget, event, data=None):
        self.response(gtk.RESPONSE_CANCEL)
        return True

    def key_press_event(self,event,key):
        if gtk.gdk.keyval_name(key.keyval) == "Escape":
            self.response(gtk.RESPONSE_CANCEL)
        return

if __name__ == "__main__":
    pref = Preferences()
    pref.run()
