#!/usr/bin/env python
# GNU GPL

import pygtk
pygtk.require('2.0')
import gtk
import gobject

class SelectListStore(gtk.ListStore):
    def __init__(self,treeview,list):
        gtk.ListStore.__init__(self, gobject.TYPE_STRING)
        self.selectSelection=treeview.get_selection()
        self.selectSelection.set_mode(gtk.SELECTION_MULTIPLE)
        self.selectSelection.select_path(0)
        self.clear()
        for row in list:
            self.append([row])
        return

    def get_selection(self):
        selection=list()
        (model,paths)=self.selectSelection.get_selected_rows()
        for selec in paths:
            iter=model.get_iter(selec[0])
            selection.append(model.get_value(iter,0))
        return selection

class Select(gtk.ScrolledWindow):
    def __init__(self,cell,name,filter_list,config):
        gtk.ScrolledWindow.__init__(self)
        
        self.treeview = gtk.TreeView()
        self.selectListStore=SelectListStore(self.treeview,filter_list)
        self.treeview.set_model(self.selectListStore)
        self.treeview.set_headers_visible(False)

        column=gtk.TreeViewColumn(name)
        self.treeview.append_column(column)
        column.pack_start(cell, True)
        column.set_attributes(cell, text=0)
        
        self.treeview.set_enable_search(False)
        self.treeview.set_cursor(0,column)
        self.add(self.treeview)

        return
        
class Selects(gtk.VBox):
    def __init__(self,config,lists):
        gtk.VBox.__init__(self)
        
        self.config = config
        self.initialize()
        cell = gtk.CellRendererText()

        # select box
        self.select_box=dict()
        for name in config["selects"]:
            self.select_box[name]= Select( cell,
                                              _(name.capitalize()),
                                              lists[name],
                                              config)
            self.select_box[name].set_size_request(200,-1)

        # select combobox
        liststore = gtk.ListStore(gobject.TYPE_STRING)
        self.combobox = gtk.ComboBox(liststore)
        self.combobox.pack_start(cell, True)
        self.combobox.add_attribute(cell, 'text', 0)
        for select in config["selects"]:
            liststore.append([_(select.capitalize())])
        self.combobox.connect("changed",self.change_select)
        self.combobox.set_active(config["selects"].index(config["default_select"]))
        

        self.pack_start(self.combobox, False, True, 0)
        for name in config["selects"]:
            self.pack_start(self.select_box[name], True, True, 0)

        return
        
    def change_select(self, item):
        for name in self.config["selects"]:
            if _(name.capitalize()) != self.combobox.get_active_text():
                self.select_box[name].hide_all()
            else:
                self.select_box[name].show_all()
                self.current_select_box=name
                self.select_box[name].treeview.grab_focus()
        self.initialize()
        return
        
    def get_selection(self):
        selection=self.select_box[self.current_select_box].selectListStore.get_selection()
        if self.lastselection == selection:
            return None
        self.lastselection=selection
        pattern=dict()
        if _('Any') not in selection:
            pattern[self.current_select_box]=""
            for select in selection:
                pattern[self.current_select_box]=pattern[self.current_select_box]+"^"+select+"$"+'|'
            pattern[self.current_select_box]=pattern[self.current_select_box][:-1].replace('*','\*')
        return pattern  


    def initialize(self):
        self.lastselection = None
        return

    def set_focus(self):
        self.select_box[self.current_select_box].treeview.grab_focus()
        return
